==> postbio-0.1.3.tar.gz: postbio-0.1.3/postbio.c <==
/* {=================================================================
 *
 * utils.c
 * A few utility functions for PostBio
 * Author: Luis Carvalho <lexcarvalho at gmail dot com>
 * Please check copyright notice at the bottom of postbio.sql
 * $Id: postbio.c,v 1.3 2009/09/19 16:24:05 carvalho Exp $
 *
 * ==================================================================} */

#include <postgres.h>
#include <fmgr.h>

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

#if PG_VERSION_NUM < 80300
#define SET_VARSIZE(ptr, len) VARATT_SIZEP(ptr) = len
#endif

/* {=====================================================================
 *    Prototypes
 * ======================================================================} */

/* int_interval */
Datum iint_in (PG_FUNCTION_ARGS);
Datum iint_out (PG_FUNCTION_ARGS);
Datum iint_recv (PG_FUNCTION_ARGS);
Datum iint_send (PG_FUNCTION_ARGS);
Datum iint_textin (PG_FUNCTION_ARGS);
Datum iint_textout (PG_FUNCTION_ARGS);
Datum iint_low (PG_FUNCTION_ARGS);
Datum iint_high (PG_FUNCTION_ARGS);
Datum integer_to_iint (PG_FUNCTION_ARGS);
Datum intpair_to_iint (PG_FUNCTION_ARGS);
Datum iint_compare (PG_FUNCTION_ARGS);
Datum iint_lt (PG_FUNCTION_ARGS);
Datum iint_le (PG_FUNCTION_ARGS);
Datum iint_eq (PG_FUNCTION_ARGS);
Datum iint_ge (PG_FUNCTION_ARGS);
Datum iint_gt (PG_FUNCTION_ARGS);
Datum iint_left (PG_FUNCTION_ARGS);
Datum iint_overleft (PG_FUNCTION_ARGS);
Datum iint_overlap (PG_FUNCTION_ARGS);
Datum iint_overright (PG_FUNCTION_ARGS);
Datum iint_right (PG_FUNCTION_ARGS);
Datum iint_same (PG_FUNCTION_ARGS);
Datum iint_contain (PG_FUNCTION_ARGS);
Datum iint_contained (PG_FUNCTION_ARGS);
Datum g_iint_consistent (PG_FUNCTION_ARGS);
Datum g_iint_union (PG_FUNCTION_ARGS);
Datum g_iint_compress (PG_FUNCTION_ARGS);
Datum g_iint_decompress (PG_FUNCTION_ARGS);
Datum g_iint_penalty (PG_FUNCTION_ARGS);
Datum g_iint_picksplit (PG_FUNCTION_ARGS);
Datum g_iint_same (PG_FUNCTION_ARGS);
Datum iint_distance_low (PG_FUNCTION_ARGS);
Datum iint_distance_high (PG_FUNCTION_ARGS);
Datum iint_distance_center (PG_FUNCTION_ARGS);
Datum iint_distance (PG_FUNCTION_ARGS);
Datum iint_join (PG_FUNCTION_ARGS);
Datum iint_overlap (PG_FUNCTION_ARGS);
Datum iint_size (PG_FUNCTION_ARGS);
Datum iint_buffer1 (PG_FUNCTION_ARGS);
Datum iint_buffer2 (PG_FUNCTION_ARGS);
Datum iint_span (PG_FUNCTION_ARGS);
/* stree */
Datum stree_in (PG_FUNCTION_ARGS);
Datum stree_out (PG_FUNCTION_ARGS);
Datum stree_recv (PG_FUNCTION_ARGS);
Datum stree_bytea (PG_FUNCTION_ARGS);
Datum stree_send (PG_FUNCTION_ARGS);
Datum stree_build (PG_FUNCTION_ARGS);
Datum stree_text (PG_FUNCTION_ARGS);
Datum stree_subtext (PG_FUNCTION_ARGS);
Datum maxmatch (PG_FUNCTION_ARGS);
Datum maxmatchcount (PG_FUNCTION_ARGS);
Datum maxmatch0 (PG_FUNCTION_ARGS);
/* utils */
Datum revcomp (PG_FUNCTION_ARGS);

/* {=====================================================================
 *    Functions
 * ======================================================================} */

/* Watson-Crick complement */
static char wccomp (char c) {
  switch (c) {
    case 'a': return 't';
    case 'c': return 'g';
    case 'g': return 'c';
    case 't': return 'a';
    case 'r': return 'y'; /* purine: a or g */
    case 'y': return 'r'; /* pyrimidine: c or t */
    case 's': case 'w': return c; /* strong (c or g) or weak (a or t) */
    case 'm': return 'k'; /* a or c */
    case 'k': return 'm'; /* g or t */
    case 'b': return 'v'; /* c, g or t */
    case 'd': return 'h'; /* a, g or t */
    case 'h': return 'd'; /* a, c or t */
    case 'v': return 'b'; /* a, c or g */
    default: return 'n';
  }
}

PG_FUNCTION_INFO_V1(revcomp);
Datum revcomp (PG_FUNCTION_ARGS) {
  text *str = PG_GETARG_TEXT_P(0);
  bool comp;
  text *rstr;
  char *ps, *pr;
  size_t l = VARSIZE(str) - VARHDRSZ;
  rstr = (text *) palloc(VARSIZE(str));
  SET_VARSIZE(rstr, VARSIZE(str));
  ps = VARDATA(str);
  pr = VARDATA(rstr);
  comp = PG_ARGISNULL(1);
  if (!comp) comp = PG_GETARG_BOOL(1);
  if (comp)
    while (l--) *pr++ = wccomp(ps[l]);
  else
    while (l--) *pr++ = ps[l];
  PG_RETURN_TEXT_P(rstr);
}

==> postbio-0.2.tar.gz: postbio-0.2/postbio.c <==
/* {=================================================================
 *
 * utils.c
 * A few utility functions for PostBio
 * Author: Luis Carvalho <lexcarvalho at gmail dot com>
 * Please check copyright notice at the bottom of postbio.sql
 *
 * ==================================================================} */

#include <postgres.h>
#include <fmgr.h>

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

#if PG_VERSION_NUM < 80300
#define SET_VARSIZE(ptr, len) VARATT_SIZEP(ptr) = len
#endif

/* {=====================================================================
 *    Prototypes
 * ======================================================================} */

/* int_interval */
Datum iint_in (PG_FUNCTION_ARGS);
Datum iint_out (PG_FUNCTION_ARGS);
Datum iint_recv (PG_FUNCTION_ARGS);
Datum iint_send (PG_FUNCTION_ARGS);
Datum iint_textin (PG_FUNCTION_ARGS);
Datum iint_textout (PG_FUNCTION_ARGS);
Datum iint_low (PG_FUNCTION_ARGS);
Datum iint_high (PG_FUNCTION_ARGS);
Datum integer_to_iint (PG_FUNCTION_ARGS);
Datum intpair_to_iint (PG_FUNCTION_ARGS);
Datum iint_compare (PG_FUNCTION_ARGS);
Datum iint_lt (PG_FUNCTION_ARGS);
Datum iint_le (PG_FUNCTION_ARGS);
Datum iint_eq (PG_FUNCTION_ARGS);
Datum iint_ge (PG_FUNCTION_ARGS);
Datum iint_gt (PG_FUNCTION_ARGS);
Datum iint_left (PG_FUNCTION_ARGS);
Datum iint_overleft (PG_FUNCTION_ARGS);
Datum iint_overlap (PG_FUNCTION_ARGS);
Datum iint_overright (PG_FUNCTION_ARGS);
Datum iint_right (PG_FUNCTION_ARGS);
Datum iint_same (PG_FUNCTION_ARGS);
Datum iint_contain (PG_FUNCTION_ARGS);
Datum iint_contained (PG_FUNCTION_ARGS);
Datum g_iint_consistent (PG_FUNCTION_ARGS);
Datum g_iint_union (PG_FUNCTION_ARGS);
Datum g_iint_compress (PG_FUNCTION_ARGS);
Datum g_iint_decompress (PG_FUNCTION_ARGS);
Datum g_iint_penalty (PG_FUNCTION_ARGS);
Datum g_iint_picksplit (PG_FUNCTION_ARGS);
Datum g_iint_same (PG_FUNCTION_ARGS);
Datum iint_distance_low (PG_FUNCTION_ARGS);
Datum iint_distance_high (PG_FUNCTION_ARGS);
Datum iint_distance_center (PG_FUNCTION_ARGS);
Datum iint_distance (PG_FUNCTION_ARGS);
Datum iint_join (PG_FUNCTION_ARGS);
Datum iint_overlap (PG_FUNCTION_ARGS);
Datum iint_size (PG_FUNCTION_ARGS);
Datum iint_buffer1 (PG_FUNCTION_ARGS);
Datum iint_buffer2 (PG_FUNCTION_ARGS);
Datum iint_span (PG_FUNCTION_ARGS);
/* stree */
Datum stree_in (PG_FUNCTION_ARGS);
Datum stree_out (PG_FUNCTION_ARGS);
Datum stree_recv (PG_FUNCTION_ARGS);
Datum stree_bytea (PG_FUNCTION_ARGS);
Datum stree_send (PG_FUNCTION_ARGS);
Datum stree_build (PG_FUNCTION_ARGS);
Datum stree_text (PG_FUNCTION_ARGS);
Datum stree_subtext (PG_FUNCTION_ARGS);
Datum stree_maxmatch (PG_FUNCTION_ARGS);
Datum stree_maxmatchcount (PG_FUNCTION_ARGS);
Datum stree_maxmatch0 (PG_FUNCTION_ARGS);
/* fmindex */
Datum fmindex_in (PG_FUNCTION_ARGS);
Datum fmindex_out (PG_FUNCTION_ARGS);
Datum fmindex_recv (PG_FUNCTION_ARGS);
Datum fmindex_bytea (PG_FUNCTION_ARGS);
Datum fmindex_send (PG_FUNCTION_ARGS);
Datum fmindex_build (PG_FUNCTION_ARGS);
Datum fmindex_text (PG_FUNCTION_ARGS);
Datum fmindex_extract (PG_FUNCTION_ARGS);
Datum fmindex_length (PG_FUNCTION_ARGS);
Datum fmindex_count (PG_FUNCTION_ARGS);
Datum fmindex_locate (PG_FUNCTION_ARGS);
/* utils */
Datum revcomp (PG_FUNCTION_ARGS);

/* {=====================================================================
 *    Functions
 * ======================================================================} */

/* Watson-Crick complement */
static char wccomp (char c) {
  switch (c) {
    case 'a': return 't';
    case 'c': return 'g';
    case 'g': return 'c';
    case 't': return 'a';
    case 'r': return 'y'; /* purine: a or g */
    case 'y': return 'r'; /* pyrimidine: c or t */
    case 's': case 'w': return c; /* strong (c or g) or weak (a or t) */
    case 'm': return 'k'; /* a or c */
    case 'k': return 'm'; /* g or t */
    case 'b': return 'v'; /* c, g or t */
    case 'd': return 'h'; /* a, g or t */
    case 'h': return 'd'; /* a, c or t */
    case 'v': return 'b'; /* a, c or g */
    default: return 'n';
  }
}

PG_FUNCTION_INFO_V1(revcomp);
Datum revcomp (PG_FUNCTION_ARGS) {
  text *str = PG_GETARG_TEXT_P(0);
  bool comp;
  text *rstr;
  char *ps, *pr;
  size_t l = VARSIZE(str) - VARHDRSZ;
  rstr = (text *) palloc(VARSIZE(str));
  SET_VARSIZE(rstr, VARSIZE(str));
  ps = VARDATA(str);
  pr = VARDATA(rstr);
  comp = PG_ARGISNULL(1);
  if (!comp) comp = PG_GETARG_BOOL(1);
  if (comp)
    while (l--) *pr++ = wccomp(ps[l]);
  else
    while (l--) *pr++ = ps[l];
  PG_RETURN_TEXT_P(rstr);
}
